SystemOrganization addCategory: #'SqSync-Shared'!
SystemOrganization addCategory: #'SqSync-Client'!
SystemOrganization addCategory: #'SqSync-Server'!
SystemOrganization addCategory: #'SqSync-Tests'!

OrderedCollection subclass: #SySharedList
	instanceVariableNames: 'mutex'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqSync-Shared'!

!SySharedList commentStamp: 'mt 5/9/2010 17:04:27.753' prior: 0!
This is meant to be a thread-safe version of an OrderedCollection. See the "list" protocol to explore selectors which are thread-safe.!

!SySharedList methodsFor: 'list' stamp: 'mt 5/9/2010 16:52:24.122'!
add: anElement

	^ self mutex critical: [super add: anElement].! !

!SySharedList methodsFor: 'accessing - private' stamp: 'mt 5/9/2010 16:40:52.25'!
mutex

	^ mutex ifNil: [mutex := Mutex new]! !

!SySharedList methodsFor: 'list' stamp: 'mt 5/9/2010 16:53:19.735'!
remove: anElement ifAbsent: aBlock
	"#remove: should also be guarded now."
	
	^ self mutex critical: [super remove: anElement ifAbsent: aBlock].! !

Object subclass: #Person
	instanceVariableNames: 'givenName surname birthday address'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqSync-Tests'!

!Person class methodsFor: 'as yet unclassified' stamp: 'SR 5/15/2010 12:28:45.274'!
givenName: aFirstString surname: aSecondString

	^ (self new)
		givenName: aFirstString;
		surname: aSecondString;
		yourself.! !

!Person methodsFor: 'as yet unclassified' stamp: 'SR 5/15/2010 15:24:16.133'!
= anObject 

	^ (self givenName = anObject givenName) and: [self surname = anObject surname] and: [self birthday = anObject birthday] and: [self address = anObject address]! !

!Person methodsFor: 'accessing' stamp: 'SR 5/15/2010 12:25:08.009'!
address
	^ address! !

!Person methodsFor: 'accessing' stamp: 'SR 5/15/2010 12:26:51.253'!
address: aString

	(aString isString) ifFalse: [Exception signal: 'Invalid data type'].
	address := aString! !

!Person methodsFor: 'accessing' stamp: 'SR 5/15/2010 12:25:18.253'!
birthday
	^ birthday! !

!Person methodsFor: 'accessing' stamp: 'SR 5/15/2010 12:27:34.621'!
birthday: aString

	(aString isString) ifFalse: [Exception signal: 'Invalid data type'].
	birthday := aString! !

!Person methodsFor: 'accessing' stamp: 'SR 5/15/2010 12:25:24.381'!
givenName
	^ givenName! !

!Person methodsFor: 'accessing' stamp: 'SR 5/15/2010 12:26:32.385'!
givenName: aString

	(aString isString) ifFalse: [Exception signal: 'Invalid data type'].
	givenName := aString! !

!Person methodsFor: 'accessing' stamp: 'SR 5/15/2010 12:25:32.002'!
surname
	^ surname! !

!Person methodsFor: 'accessing' stamp: 'SR 5/15/2010 12:26:22.555'!
surname: aString

	(aString isString) ifFalse: [Exception signal: 'Invalid data type'].
	surname := aString.! !

Object subclass: #SyProtocolProcessor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqSync-Shared'!

!SyProtocolProcessor methodsFor: 'as yet unclassified' stamp: 'SR 5/15/2010 12:22:17.087'!
processDataFrom: aStream
	"Hint: Expect stream API from SocketStream. Other Streams should work as well."
	self subclassResponsibility! !

SyProtocolProcessor subclass: #SySimpleProtocolProcessor
	instanceVariableNames: 'server'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqSync-Shared'!

!SySimpleProtocolProcessor class methodsFor: 'as yet unclassified' stamp: 'SR 5/15/2010 13:03:35.969'!
on: aServer

	^ self new server: aServer.! !

!SySimpleProtocolProcessor methodsFor: 'as yet unclassified' stamp: 'SR 5/15/2010 15:13:53.725'!
processDataFrom: aStream
	"For now do nothing."
	
	| command |
	command := aStream upToAll: String crlf.
	(command = 'list') ifTrue: [ | objects data |
		"Execute list command: stream all objects to client."
		objects := Dictionary new.
		self server objectDictionary keysAndValuesDo: [ :key :object | objects at: key put: (self pvtCreateObjectDictionary: object) ].
		data := (Json render: objects), String crlf.

		aStream nextPutAllFlush: data; crlf.
		^ true
	].
	(command = 'quit') ifTrue: [
		^ false
	].
		! !

!SySimpleProtocolProcessor methodsFor: 'as yet unclassified' stamp: 'SR 5/15/2010 15:13:36.155'!
pvtCreateObjectDictionary: anObject

	| result |
	result := Dictionary new.
	result at: 'type' put: (anObject className).
	result at: 'data' put: (anObject instanceVariableValues).
	^ result! !

!SySimpleProtocolProcessor methodsFor: 'as yet unclassified' stamp: 'SR 5/15/2010 13:03:12.643'!
server

	^ server! !

!SySimpleProtocolProcessor methodsFor: 'as yet unclassified' stamp: 'SR 5/15/2010 13:03:21.002'!
server: aServer

	server := aServer! !

Object subclass: #SySimpleClient
	instanceVariableNames: 'socket objectDictionary'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqSync-Client'!

!SySimpleClient methodsFor: 'connection-handling' stamp: 'SR 5/15/2010 14:52:28.811'!
connectToHostNamed: aHost port: aPort

	socket := SyTcpSocket openConnectionToHostNamed: aHost port: aPort.! !

!SySimpleClient methodsFor: 'connection-handling' stamp: 'SR 5/15/2010 14:45:47.449'!
disconnect

	socket close.! !

!SySimpleClient methodsFor: 'initialize-release' stamp: 'SR 5/15/2010 14:49:22.066'!
initialize

	super initialize.! !

!SySimpleClient methodsFor: 'connection-handling' stamp: 'SR 5/15/2010 14:50:21.163'!
isConnected

	socket ifNil: [ ^ false ].
	^ socket isConnected! !

!SySimpleClient methodsFor: 'managed-objects' stamp: 'SR 5/15/2010 14:56:56.944'!
managedObjects

	objectDictionary ifNil: [self pvtUpdateManagedObjects].
	^ objectDictionary values! !

!SySimpleClient methodsFor: 'managed-objects' stamp: 'SR 5/15/2010 15:21:13.547'!
pvtUpdateManagedObjects

	| response rawResponse |
	objectDictionary := Dictionary new.

	"Send request"
	socket nextPutAllFlush: ('list', String crlf).
	rawResponse := socket upToAll: String crlf.
	response := Json readFrom: (ReadStream on: rawResponse).
	response keysAndValuesDo: [:key :data | | typeName variableData newObject |
		typeName := data at: 'type'.
		variableData := data at: 'data'.
		
		newObject := (Smalltalk at: (typeName asSymbol)) new.
		variableData withIndexDo: [:value :index | newObject instVarAt: index put: value].
		
		objectDictionary at: key put: newObject.
	].! !

Object subclass: #SyTcpServer
	instanceVariableNames: 'listenSocket listenProcess connections connectionProcesses connectionChangesMonitor protocolProcessor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqSync-Server'!

SyTcpServer subclass: #SySimpleServer
	instanceVariableNames: 'objectDictionary'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqSync-Server'!

!SySimpleServer methodsFor: 'initialize-release' stamp: 'SR 5/15/2010 13:03:51.836'!
initialize

	super initialize.
	
	protocolProcessor := SySimpleProtocolProcessor on: self.
	objectDictionary := Dictionary new.! !

!SySimpleServer methodsFor: 'managed-objects' stamp: 'SR 5/15/2010 12:11:13.094'!
managedObjects

	^ objectDictionary values! !

!SySimpleServer methodsFor: 'managed-objects' stamp: 'SR 5/15/2010 13:14:53.346'!
objectDictionary

	^ objectDictionary! !

!SySimpleServer methodsFor: 'managed-objects' stamp: 'SR 5/15/2010 15:17:58.635'!
register: anObject

	(self managedObjects contains: [ :o | o == anObject]) ifTrue: [ Exception signal: 'Object already registered' ].
	objectDictionary at: (UUID new asString) put: anObject.! !

!SySimpleServer methodsFor: 'managed-objects' stamp: 'SR 5/15/2010 12:20:12.965'!
unregister: anObject

	objectDictionary keysAndValuesRemove: [ :uuid :object | object == anObject ]! !

!SyTcpServer methodsFor: 'connection open/close' stamp: 'rkrk 5/14/2010 17:50'!
close

	"Closing listen part"
	listenProcess terminate.
	self listenSocket close.
	listenSocket := nil.
	
	"Closing connection part"
	self connectionProcesses do: [:proc | proc terminate].
	self connections do: [:ea | ea close].! !

!SyTcpServer methodsFor: 'waiting' stamp: 'SR 5/10/2010 18:25:52.291'!
connectionChangesMonitor

	^ connectionChangesMonitor! !

!SyTcpServer methodsFor: 'accessing - private' stamp: 'SR 5/10/2010 17:28:42.275'!
connectionProcesses

	^ connectionProcesses ifNil: [connectionProcesses := SySharedList new]! !

!SyTcpServer methodsFor: 'accessing - private' stamp: 'mt 5/9/2010 16:46:22.708'!
connections

	^ connections ifNil: [connections := SySharedList new]! !

!SyTcpServer methodsFor: 'synchronizing' stamp: 'SR 5/15/2010 13:53:12.092'!
connectionsChanged

	"[self connectionChangesMonitor signalAll]
		on: Error do: [:e |
			self
				log: 'Problem signaling connectionChangesMonitor';
				log: e.
			self halt.]."! !

!SyTcpServer methodsFor: 'waiting' stamp: 'SR 5/10/2010 18:26:19.094'!
initialize

	super initialize.
	
	connectionChangesMonitor := Monitor new.! !

!SyTcpServer methodsFor: 'testing' stamp: 'mt 5/9/2010 16:17:23.888'!
isListening

	^ self listenSocket isWaitingForConnection! !

!SyTcpServer methodsFor: 'connection open/close' stamp: 'rkrk 5/14/2010 13:30'!
listenOn: aPort

	self isListening ifTrue: [self error: 'Server already listening!!'].
	self listenSocket listenOn: aPort backlogSize: 4.
	self startListenThread.! !

!SyTcpServer methodsFor: 'accessing - private' stamp: 'mt 5/9/2010 15:58:25.953'!
listenSocket

	^ listenSocket ifNil: [listenSocket := Socket newTCP]! !

!SyTcpServer methodsFor: 'debugging' stamp: 'rkrk 5/14/2010 17:31'!
log: msg

	Transcript cr; show: msg asString.! !

!SyTcpServer methodsFor: 'accessing - private' stamp: 'rkrk 5/14/2010 13:47'!
newConnectionOn: socket

	| con |
	con := SyTcpSocket on: socket.
	self connections add: con.
	self startConnectionThreadFor: con.
! !

!SyTcpServer methodsFor: 'accessing' stamp: 'mt 5/9/2010 17:12:59.484'!
port

	^ self listenSocket port! !

!SyTcpServer methodsFor: 'threads' stamp: 'SR 5/15/2010 14:16:47.584'!
startConnectionThreadFor: aTcpSocket

	| connectionProcess |
	connectionProcess := [
		[aTcpSocket isConnected] whileTrue: [
			aTcpSocket waitForDataIfClosed: [
				self connections remove: aTcpSocket.
				self log: 'Some client socket closed'; log: aTcpSocket.
				self connectionsChanged
			].
			(protocolProcessor processDataFrom: aTcpSocket) ifFalse: [
				aTcpSocket close.
				self connections remove: aTcpSocket.
				self log: 'Some client socket closed'; log: aTcpSocket.
				self connectionsChanged
			]
		].
	].
	self connectionProcesses add: (connectionProcess forkAt: 35).! !

!SyTcpServer methodsFor: 'threads' stamp: 'SR 5/15/2010 12:36:03.582'!
startListenThread
	
	listenProcess := [
		[self isListening] whileTrue: [
				(self listenSocket waitForAcceptFor: 10) ifNotNilDo: [:socket |
					self newConnectionOn: socket.
					self log: 'New client connected'; log: socket.
					self connectionsChanged]]] forkAt: 35.! !

!SyTcpServer methodsFor: 'waiting' stamp: 'mt 5/10/2010 11:27:21.847'!
waitForNewConnectionFrom: nameOrAddressString port: aPort

	| remoteAddress |
	remoteAddress := NetNameResolver addressForName: nameOrAddressString timeout: 20.
	
	[
		self connections includes: [:tcpSocket | 
			tcpSocket remoteAddress = remoteAddress
			and: [tcpSocket remotePort = aPort]].
		Processor yield
	] whileFalse.! !

TestCase subclass: #SyCommunicationModuleTest
	instanceVariableNames: 'tcpServer tcpSocket openClientSockets'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqSync-Tests'!

!SyCommunicationModuleTest class methodsFor: 'helper' stamp: 'rkrk 5/14/2010 17:02'!
closeAllSockets

	Socket registry keys
		select: [:ea | ea isWaitingForConnection or: [ea isOtherEndClosed] or: [ea isConnected]]
		thenDo: [:ea | ea close].! !

!SyCommunicationModuleTest class methodsFor: 'constants' stamp: 'rkrk 5/14/2010 14:24'!
defaultPort

	^ 22701! !

!SyCommunicationModuleTest methodsFor: 'support' stamp: 'rkrk 5/14/2010 18:19'!
connectTCPClientToServer

	tcpSocket := SyTcpSocket
		connectToHost: '127.0.0.1'
		port: self class defaultPort.
	openClientSockets add: tcpSocket.
	^ tcpSocket.! !

!SyCommunicationModuleTest methodsFor: 'support' stamp: 'rkrk 5/14/2010 18:15'!
disconnectTCPClientsFromServer

	tcpSocket ifNotNil: [tcpSocket close. tcpSocket := nil].
	openClientSockets do: [:ea | ea close.]! !

!SyCommunicationModuleTest methodsFor: 'support' stamp: 'SR 5/15/2010 12:46:22.685'!
maybeWaitForServerThenDo: block

	"sometimes the server is already done before waiting is started so try the block
	first then optionally wait"
	tcpServer connectionChangesMonitor critical: [
		block on: Exception do: [
			Transcript cr;
				show: 'maybeWaitForServerThenDo: first try was failure in ';
					show: self selector.
			tcpServer connectionChangesMonitor wait.
			block value]].! !

!SyCommunicationModuleTest methodsFor: 'running' stamp: 'rkrk 5/14/2010 18:17'!
setUp

	super setUp.
	openClientSockets := OrderedCollection new.
	Transcript cr; show: 'start '; show: self selector.! !

!SyCommunicationModuleTest methodsFor: 'support' stamp: 'mt 5/10/2010 11:39:41.976'!
startTcpServer

	tcpServer ifNil: [tcpServer := SyTcpServer new].
	tcpServer isListening
		ifFalse: [tcpServer listenOn: self class defaultPort].! !

!SyCommunicationModuleTest methodsFor: 'support' stamp: 'SR 5/10/2010 17:17:02.712'!
stopTcpServer

	(tcpServer notNil and: [tcpServer isListening])
		ifTrue: [tcpServer close].! !

!SyCommunicationModuleTest methodsFor: 'running' stamp: 'rkrk 5/14/2010 18:15'!
tearDown

	super tearDown.
	Transcript cr; show: 'end '; show: self selector.
	[self
		disconnectTCPClientsFromServer;
		stopTcpServer
	] on: Error do: [:e | Transcript cr; show: 'Error in tearDown'; print: e].! !

!SyCommunicationModuleTest methodsFor: 'tests' stamp: 'rkrk 5/14/2010 14:23'!
test01TcpServerStartup

	| sut |
	sut := SyTcpServer new.
	
	self deny: sut isListening.
	
	[sut listenOn: self class defaultPort.
	self
		assert: sut isListening;
		assert: self class defaultPort equals: sut port;
		assert: sut connections isEmpty.
	] ensure: [sut close].! !

!SyCommunicationModuleTest methodsFor: 'tests' stamp: 'rkrk 5/14/2010 14:24'!
test02TcpServerShutdown
	
	self
		startTcpServer;
		stopTcpServer;
		deny: tcpServer isListening;
		assert: tcpServer connections isEmpty;
		assert: (Socket registry keys noneSatisfy: [:socket | socket isWaitingForConnection]).! !

!SyCommunicationModuleTest methodsFor: 'tests' stamp: 'SR 5/15/2010 12:55:44.833'!
test03FailingTcpSocketConnection

	"try to connect to a server. This should either throw a ConnectionTimedOut error.
	Since we currently cannot control the timeout period of sockets and the default of 
	[Socket standardTimeout] = 45 is used we end the test earlier and just assume
	that the connection can't be established."
	<timeout: 6> "1s more than needed"
	| connectionFailed connectReturned connectProcess |
	connectionFailed := false.
	connectReturned := false.
	connectProcess := [
		[self connectTCPClientToServer. connectReturned := true]
			on: ConnectionTimedOut
			do: [connectionFailed := true]] fork.
	self wait: 5. "seconds"
	connectProcess terminate.
	connectionFailed ifTrue: [^self].
	connectReturned ifTrue: [^self]. "assume it works"
	self
		assert: false
		description: '#connectTCPClientToServer was successful although no server was started!!'.! !

!SyCommunicationModuleTest methodsFor: 'tests' stamp: 'SR 5/15/2010 12:44:53.636'!
test04TcpSocketConnection

	<timeout: 5>
	self
		startTcpServer;
		connectTCPClientToServer;
		assert: tcpSocket isConnected;
		maybeWaitForServerThenDo: [self assert: 1 equals: tcpServer connectionProcesses size].! !

!SyCommunicationModuleTest methodsFor: 'tests' stamp: 'SR 5/15/2010 12:55:36.418'!
test05TcpServerIncomingConnections

	self
		startTcpServer;
		assert: tcpServer connections isEmpty;
		connectTCPClientToServer;
		maybeWaitForServerThenDo: [self assert: tcpServer connections size = 1].

	self connectTCPClientToServer.
		
	self
		maybeWaitForServerThenDo: [self assert: tcpServer connections size = 2];
		disconnectTCPClientsFromServer;
		assert: tcpServer connections size = 1.
	
	self assert: tcpServer connections isEmpty.! !

!SyCommunicationModuleTest methodsFor: 'support' stamp: 'rkrk 5/14/2010 16:55'!
wait: seconds

	(Delay forSeconds: seconds) wait.! !

TestCase subclass: #SySimpleClientGetObjectTest
	instanceVariableNames: 'server o1 o2 sut'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqSync-Tests'!

!SySimpleClientGetObjectTest class methodsFor: 'as yet unclassified' stamp: 'SR 5/15/2010 14:41:32.817'!
defaultPort

	^ 22701! !

!SySimpleClientGetObjectTest methodsFor: 'running' stamp: 'SR 5/15/2010 14:30:22.383'!
setUp

	server := SySimpleServer new.
	o1 := Person givenName: 'Micha' surname: 'Test'.
	o2 := Person givenName: 'Christoph' surname: 'Muster'.
	server
		register: o1;
		register: o2;
		listenOn: self class defaultPort.
		
	sut := SySimpleClient new.! !

!SySimpleClientGetObjectTest methodsFor: 'running' stamp: 'SR 5/15/2010 14:31:06.456'!
tearDown

	server close.! !

!SySimpleClientGetObjectTest methodsFor: 'testing' stamp: 'SR 5/15/2010 14:52:42.972'!
test01ConnectDisconnect

	sut connectToHostNamed: '127.0.0.1' port: self class defaultPort.
	self assert: sut isConnected.
	sut disconnect.! !

!SySimpleClientGetObjectTest methodsFor: 'testing' stamp: 'SR 5/15/2010 14:52:48.597'!
test02GetObjects

	sut connectToHostNamed: '127.0.0.1' port: self class defaultPort.
	self assert: sut isConnected.
	
	self assert: (sut managedObjects size = 2).
	self assert: (sut managedObjects contains: [:o| o = o1]).
	self assert: (sut managedObjects contains: [:o| o = o2]).
	
	
	sut disconnect.! !

TestCase subclass: #SySimpleServerObjectRepoTests
	instanceVariableNames: 'sut'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqSync-Tests'!

!SySimpleServerObjectRepoTests methodsFor: 'running' stamp: 'SR 5/15/2010 12:08:17.176'!
setUp

	super setUp.
	sut := SySimpleServer new.! !

!SySimpleServerObjectRepoTests methodsFor: 'testing' stamp: 'SR 5/15/2010 12:20:41.821'!
test01UnRegisterObject

	| o1 |
	o1 := Person new.
	self assert: (sut managedObjects isEmpty).
	sut register: o1.
	self assert: (sut managedObjects size = 1).
	sut unregister: o1.
	self assert: (sut managedObjects isEmpty).! !

!SySimpleServerObjectRepoTests methodsFor: 'testing' stamp: 'SR 5/15/2010 12:18:28.299'!
test02DoubleRegistration

	| o1 alreadyRegisteredException |
	o1 := Person new.
	self assert: (sut managedObjects isEmpty).
	sut register: o1.
	self assert: (sut managedObjects size = 1).
	
	alreadyRegisteredException := false.
	[sut register: o1]
		on: Exception
		do: [ alreadyRegisteredException := true ].
	self assert: alreadyRegisteredException.
! !

SocketStream subclass: #SyTcpSocket
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqSync-Shared'!

!SyTcpSocket class methodsFor: 'instance creation' stamp: 'mt 5/10/2010 11:05:32.461'!
connectToHost: nameOrAddressString port: anInteger
	
	^ self
		openConnectionToHostNamed: nameOrAddressString
		port: anInteger! !

!SyTcpSocket methodsFor: 'accessing' stamp: 'mt 5/10/2010 11:23:48.023'!
remoteAddress

	^ self socket remoteAddress! !

!SyTcpSocket methodsFor: 'accessing' stamp: 'mt 5/10/2010 11:23:53.871'!
remotePort

	^ self socket remotePort! !

!SyTcpSocket methodsFor: 'waiting' stamp: 'mt 5/9/2010 17:23:15.848'!
waitForDataIfClosed: aBlock

	^ self socket waitForDataIfClosed: aBlock! !
